<?php

// Připojíme potřebné soubory
require_once 'Menu.php';

/**
 * FrontController - ridi zivotni cyklus aplikace
 *
 * @author Admin
 */
class FrontController {
    
    // Nestatické vlastnosti třídy
    // Obsahují tzv. modifikátory přístupu (private, protected, public)
    
    /**
     * Obsahuje relativní cestu k adresáři, kde leží datové soubory s HTML
     * budeme jej nastavovat pomocí metody konstruktor
     * @var string 
     */
    private $dataFileDir;    
    
    /**
     * Obsahuje řetězec znaků odpovídající nazvu parametru získaného z URL
     * @example page - budeme jej nastavovat pomocí metody konstruktor
     * @var string 
     */
    private $pageParamName;    
    
    /**
     * Obsahuje řetezec znaků hodnoty parametru získaného z URL
     * @example titulka | informace | kontakt etc
     * @var string 
     */
    private $pageParamValue;
       
    /**
     * Obsahuje kompletní relativní cestu k datovému souboru
     * @example ./data/titulka.php 
     * @var string 
     */
    private $dataFilePath;
    
    /**
     * Obsahuje objekt menu
     * @var object Menu 
     */
    public $menu;
    
    
    // Dále definujeme metody třídy
    // Dělíme je na magické a klasické
    // Magické metody jsou aut. volány nějakou akcí
    // Jejich název začíná dvojím podtržítkem __
    
    
    /**
     * Konstruktor - magická metody aut. volána při vytvoření objektu z třídy
     * V konstruktoru můžeme nastavit vlastnosti ze vstrupních parametrů
     * @param string $dataFileDir
     * @param string $pageParamName
     */
    public function __construct($dataFileDir, $pageParamName) 
    {
        // Následující 2 řády znamenají:
        // Nestatiká vlastnost třídy = hodnotaVstupnihoParametru (do vlastnosti tedy ukládám hodnotu parametru)
        // Použijeme klasický operátor =
        $this->dataFileDir = $dataFileDir;
        $this->pageParamName = $pageParamName;
    }
    
    
    /**
     * Klasická metoda třídy
     * Řídí chod aplikace
     */
    public function run() 
    {
        // Do lokální proměnné načtu hodnotu parametru z URL
        // Použiji k tomu členskou fci PHP filter_input
        $pageParamValue = \filter_input(INPUT_GET, $this->pageParamName);
        
        // Ošetřím možné stavy lokální proměnné podle možností fce filter_input
        // Funkce filter_input selhala
        if ($pageParamValue === FALSE) {
            // Do hodonoty parametru z URL si proto uložíme výchozí datový sooubor
            $this->pageParamValue = Config::DEFAULT_DATAFILE;
        }
        // Parametry se zadaným názvem v URL nnexistruje
        elseif ($pageParamValue === NULL) {
            // Do hodonoty parametru z URL si proto uložíme výchozí datový sooubor
            $this->pageParamValue = Config::DEFAULT_DATAFILE;
        }
        // filter_input vrací hodnotu
        else {
            $this->pageParamValue = $pageParamValue;
        }
        
        // Volám pomocné metody ve správném pořadí
        $this->setDataFilePath();
        // Pridame menu aplikace
        $this->setMenu();
        $this->render();
    }
    
    
    /**
     * Nastavuje vlastnost dataFilePath
     * Je to privátní metoda - lze ji tedy volát pouze uvnitř vlastní třídy
     */
    private function setDataFilePath() 
    {
        // Lokální proměnná vznikne spojení řetězců pomocí operátoru '.'
        // napr. ./data/titulka.php 
        $dataFilePath = $this->dataFileDir . $this->pageParamValue . ".php";
        
        // Oveříme existenci datového souboru
        if (\file_exists($dataFilePath)) {
            $this->dataFilePath = $dataFilePath;
        }
        else {
            $this->dataFilePath = NULL;
        }
    }
    
    
    /**
     * Nastavuje menu aplikace
     */
    public function setMenu() 
    {
        $this->menu = new Menu(); // Vytvorim objekt tridy Menu
        
        // Naplnim objekt menu polozkamu
        foreach (Config::$myItems as $fileName => $title) {
            $this->menu->addItem($fileName, $title);
        }
        
        // Nastavim aktivni polozku (prave prohlizenou v prohlizeci
        $this->menu->setActiveItem($this->pageParamValue);
    }
    
    
    /**
     * Metoda která končí životní cyklus této třídy
     * Vykreslí HTML a výstup pošle na klienta do jeho prohližeče
     */
    private function render() 
    {
        // Uložíme si objekt menu do lokální proměnné pro snadnější práci v souboru HTML (head.inc)
        $menu = $this->menu;
        
        // Uložíme název parameteru do lok. proměnné
        $pageParamName = $this->pageParamName;
        
        // Hlavička
        require_once './inc/head.inc';
        
        // Datový soubor
        if ($this->dataFilePath !== NULL) {
            require_once $this->dataFilePath;
        }
        else {
            require_once '404.php';
        }
        
        // Patička
        require_once './inc/bottom.inc';
    }

}
